@@ -74,10 +74,12 @@ struct status_block {
     char *name;
     char *instance;
 
-    TAILQ_ENTRY(status_block) blocks;
+    TAILQ_ENTRY(status_block)
+    blocks;
 };
 
-TAILQ_HEAD(statusline_head, status_block) statusline_head;
+TAILQ_HEAD(statusline_head, status_block)
+statusline_head;
 
 #include "child.h"
 #include "ipc.h"
