@@ -22,6 +22,25 @@
  *
  */
 int ipc_connect(const char *socket_path) {
+    char *path = NULL;
+    if (socket_path != NULL) {
+        path = sstrdup(socket_path);
+    }
+
+    if (path == NULL) {
+        if ((path = getenv("I3SOCK")) != NULL) {
+            path = sstrdup(path);
+        }
+    }
+
+    if (path == NULL) {
+        path = root_atom_contents("I3_SOCKET_PATH", NULL, 0);
+    }
+
+    if (path == NULL) {
+        path = sstrdup("/tmp/i3-ipc.sock");
+    }
+
     int sockfd = socket(AF_LOCAL, SOCK_STREAM, 0);
     if (sockfd == -1)
         err(EXIT_FAILURE, "Could not create socket");
@@ -31,9 +50,9 @@ int ipc_connect(const char *socket_path) {
     struct sockaddr_un addr;
     memset(&addr, 0, sizeof(struct sockaddr_un));
     addr.sun_family = AF_LOCAL;
-    strncpy(addr.sun_path, socket_path, sizeof(addr.sun_path) - 1);
+    strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
     if (connect(sockfd, (const struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0)
-        err(EXIT_FAILURE, "Could not connect to i3");
-
+        err(EXIT_FAILURE, "Could not connect to i3 on socket %s", path);
+    free(path);
     return sockfd;
 }
